@@ -1,5 +1,5 @@
 // Copyright (c) 1995  David Engberg  All rights reserved
-// $Id: Compiler.h,v 1.15 1997/03/02 22:53:42 geppetto Exp $
+// $Id: Compiler.h,v 1.19 1997/11/10 00:48:06 geppetto Exp $
 #ifndef _Compiler_h
 #define _Compiler_h
 #pragma interface
@@ -21,9 +21,11 @@ class CIntermediateClass;
 class CIntermediateFunction;
 class CCompoundStatement;
 class CDeclarationStatement;
+class COuterLocalExpression;
 class CStatement;
 class CCompileError;
 class CJavaFieldInfo;
+class CLocalVariableRecord;
 class ostream;
 
 
@@ -55,36 +57,52 @@ public:
   void StartClass(const unicode_string& className,
 		  CJavaAccessFlags* adoptModifiers = 0,
 		  unicode_string* adoptExtends = 0,
-		  deque<unicode_string>* adoptInterfaces = 0);
+		  deque<unicode_string>* adoptInterfaces = 0,
+                  bool deprecated = false);
   void StartInterface(const unicode_string& className,
 		      CJavaAccessFlags* adoptModifiers = 0,
-		      deque<unicode_string>* adoptInterfaceExtends = 0);
+		      deque<unicode_string>* adoptInterfaceExtends = 0,
+		      bool deprecated = false);
+  unicode_string StartAnonymousClass(unicode_string* adoptParentName);
+			   
   void EndClass();
   void StartFunction(const CJavaTypeSignature& returnType,
 		     const unicode_string& functionName,
 		     const CJavaAccessFlags& modifiers,
-		     deque<CJavaFieldSignature>* adoptParameters = 0,
-		     deque<unicode_string>* adoptThrows = 0);
+		     deque<CVariableDeclaration>* adoptParameters = 0,
+		     deque<unicode_string>* adoptThrows = 0,
+		     bool deprecated = false);
   void StartConstructor(const CJavaAccessFlags& modifiers,
-			deque<CJavaFieldSignature>* adoptParameters = 0,
-			deque<unicode_string>* adoptThrows = 0);
+			deque<CVariableDeclaration>* adoptParameters = 0,
+			deque<unicode_string>* adoptThrows = 0,
+			bool deprecated = false);
   void EndFunction(CCompoundStatement* adoptStatementBlock = 0);
-  bool InStatementBlock() const { return fVariableScopes.size() > 0; }
+  bool InStatementBlock() const;
   void PushLocalScope();
   void PopLocalScope();
-  unsigned short AddLocalVariable(const CJavaFieldSignature& signature);
+  unsigned short AddLocalVariable(const CVariableDeclaration& signature);
   typedef unsigned short LocalVariableIndex;
-  const pair<CJavaFieldSignature, LocalVariableIndex>*
+  const CLocalVariableRecord*
     LookupLocalVariable(const unicode_string& name) const;
+  const COuterLocalExpression*
+    LookupOuterLocalVariable(const unicode_string& name) const;
   unsigned short AddUnnamedVariable(unsigned long width = 1);
   void AddField(CDeclarationStatement* adoptDeclaration);
   void AddStaticCode(CCompoundStatement* adoptStatement);
   void ParseError(unsigned long lineNumber, const string& errorMessage,
 		  const string& input);
+  void ParseWarning(unsigned long lineNumber, const string& errorMessage,
+		    const string& input);
 
-  const CIntermediateClass* GetCurrentClass() const { return fCurrentClass; }
+  CIntermediateClass* GetCurrentClass();
+  const CIntermediateClass* GetCurrentClass() const;
+
+  const CJavaClassFile* LookupClass(const unicode_string& name,
+		    const CIntermediateClass* classContext = 0,
+		    const CIntermediateFunction* methodContext = 0) const;
+  const CJavaClassFile* LookupClass(const unicode_string& name,
+				    const CCompileContext& context) const;
 
-  const CJavaClassFile* LookupClass(const unicode_string& name) const;
   bool ValidClass(const unicode_string& name) const;
   bool ValidParent(const CJavaClassFile& child,
 		   const CJavaClassFile& parent) const;
@@ -113,7 +131,8 @@ public:
   CCompileError* CheckValidOverride(const CJavaClassFile& parentClass,
 				    const CIntermediateFunction& method);
   CCompileError* FindConstantField(CExpression*& result,
-      const unicode_string& className, const CJavaFieldSignature& field) const;
+      const unicode_string& className, const CJavaFieldSignature& field,
+      const CCompileContext& context, unsigned long lineNumber) const;
   CCompileError* FindIntermediateConstantField(CExpression*& result,
 				      const CIntermediateClass& intermediate,
 				      const CJavaFieldInfo& fieldInfo) const;
@@ -126,6 +145,12 @@ public:
   		const CJavaClassFile& fromClass,
 		const deque<CJavaTypeSignature>& arguments,
 		string*& errorString);
+  pair<const CJavaClassFile*, const CJavaMethodInfo*>*
+    MatchConstructor(const CJavaClassFile& fromClass,
+		     const deque<CJavaTypeSignature>& arguments,
+		     ExpressionList::const_iterator& visibleBegin,
+		     ExpressionList::const_iterator& visibleEnd,
+		     string*& errorString);
   const CJavaMethodInfo*
     MatchClassMethod(const CJavaClassFile& onClass,
 		     const unicode_string& name,
@@ -143,6 +168,8 @@ public:
   typedef list<CJavaMethodSignature> MethodList;
   unsigned long UnimplementedMethods(const CJavaClassFile& onClass,
 				     MethodList& methods);
+  unsigned long UndeclaredInterfaceMethods(const CJavaClassFile& onClass,
+					   MethodList& methods);
 
   bool IsThrowable(const CJavaTypeSignature& type) const;
   bool InClassInitializers() const { return fInClassInitializers; }
@@ -158,6 +185,19 @@ public:
   StatementList::iterator GetContextBegin();
   StatementList::iterator GetContextEnd();
 
+  void GenerateSyntheticConstructorArguments(const unicode_string& className,
+					     ExpressionList& into) const;
+
+  void WarnDeprecatedField(const CJavaFieldInfo& field,
+			   const CJavaClassFile& fieldClass,
+			   const CCompileContext& context,
+			   unsigned long line = 0) const;
+  void WarnDeprecatedMethod(const CJavaMethodInfo& method,
+			   const CJavaClassFile& methodClass,
+			   const CCompileContext& context,
+			   unsigned long line = 0) const;
+  void WarnDeprecatedClass(const CJavaClassFile& usedClass) const;
+
   static const unsigned long kVersionID;
   static const unicode_string kConstructorName;
   static const unicode_string kStaticName;
@@ -165,12 +205,18 @@ public:
   static const unicode_string kThrowableName;
   static const unicode_string kErrorName;
   static const unicode_string kRuntimeExceptionName;
+
+  static const unicode_string kSyntheticDivider;
+  static const unicode_string kOuterThisName;
+  static const unicode_string kSyntheticFieldPrefix;
+
   static const string kDefaultImport;
 protected:
   CCompiler();
   CCompiler(const deque<string>& classPath);
   ~CCompiler();
 
+  bool CheckJavaLangObject();
   bool ParsedFile(const string& fileName) const;
   bool ParseFile(const string& fileName);
   bool CheckImports(const string& fileName);
@@ -184,7 +230,10 @@ protected:
 
   void PrepareIntermediateClass(CIntermediateClass& intermediate,
 				const unicode_string& fileName);
+  void RemoveIntermediateClass(const CIntermediateClass* intermediate);
   bool PrepareClassDeclarations(CIntermediateClass& intermediate);
+  void AddSyntheticConstructorParams(CIntermediateClass& inClass,
+				     CIntermediateFunction& intermediate);
   bool GenerateClass(CIntermediateClass& intermediate);
   CCompileError* GenerateCode(CIntermediateClass& intermediate,
 			      CIntermediateFunction& intermediateMethod,
@@ -192,9 +241,18 @@ protected:
   bool GenerateFieldConstants(CIntermediateClass& intermediate);
   CCompileError* GenerateMethod(CIntermediateClass& intermediateClass,
 				CIntermediateFunction& intermediateMethod);
+  void GenerateSyntheticCode(CCompileContext& context,
+			     CCodeSequence& code);
+  void FixAnonymousSyntheticConstructor(CIntermediateClass& intermediateClass,
+					CIntermediateFunction& constructor,
+					const CJavaMethodInfo& parentInfo);
 
   const CIntermediateClass*
                 LookupIntermediateClass(const unicode_string& name) const;
+  const CJavaClassFile* LookupInnerClass(const unicode_string& name,
+			 const CIntermediateClass* classContext,
+			 const CIntermediateFunction* methodContext) const;
+
   void PrintCompileError(const string& fileName, const string& message,
 			 unsigned long line = 0) const;
   void PrintCompileError(const string& fileName, const unicode_string& message,
@@ -211,22 +269,18 @@ protected:
   ImportTable fImportAliases;
   unicode_string fPackageName;
   IntermediateList fIntermediateClasses;
-  CIntermediateClass* fCurrentClass;
-  CIntermediateFunction* fCurrentFunction;
   bool fParseError;
-  unsigned short fCurrentLocalVariable;
-  deque<deque<unicode_string> > fVariableScopes;
-  typedef map<unicode_string, pair<CJavaFieldSignature, LocalVariableIndex>,
-             less<unicode_string> > LocalVariableTable;
-  LocalVariableTable fLocalVariables;
-  FinallyHandlerStack fFinallyHandlers;
-  StatementList fStatementContexts;
-  bool fInClassInitializers;
   deque<string> fDependencies;
   deque<unicode_string> fClassImports;
   vector<unsigned long> fClassImportLocations;
   deque<unicode_string> fPackageImports;
   vector<unsigned long> fPackageImportLocations;
+  bool fInClassInitializers;
+  StatementList fStatementContexts;
+  FinallyHandlerStack fFinallyHandlers;
+
+  deque<CIntermediateClass*> fParsingClasses;
+
 };
 
 #endif
