@@ -1,5 +1,5 @@
 // Copyright (c) 1995  David Engberg  All rights reserved
-// $Id: Expression.h,v 1.12 1997/03/02 22:53:43 geppetto Exp $
+// $Id: Expression.h,v 1.14 1997/11/10 00:48:06 geppetto Exp $
 #ifndef _Expression_h
 #define _Expression_h
 #pragma interface
@@ -241,7 +241,8 @@ private:
 class CNewObject : public CExpression {
   DynamicCastDeclarations;
 public:
-  CNewObject(const unicode_string& className, ExpressionList* adoptArgs = 0);
+  CNewObject(const unicode_string& className, ExpressionList* adoptArgs = 0,
+	     CExpression* explicitThis = 0);
   CNewObject(const CNewObject& source);
   virtual ~CNewObject();
   virtual CExpression* Clone() const { return new CNewObject(*this); }
@@ -250,6 +251,7 @@ public:
 	           bool isStatement = false);
 private:
   ExpressionList* fConstructorArguments;
+  CExpression* fExplicitThis;
 };
 
 //
@@ -324,12 +326,13 @@ class CLocalVariableExpression : public CExpression {
   DynamicCastDeclarations;
 public:
   CLocalVariableExpression(const CJavaFieldSignature& name,
-			   unsigned short index);
+			   unsigned short index,
+			   bool final);
   CLocalVariableExpression(const CLocalVariableExpression& source);
   virtual ~CLocalVariableExpression();
   virtual CExpression* Clone() const
      { return new CLocalVariableExpression(*this); }
-  unsigned short GetVariableIndex() const;
+  unsigned short GetVariableIndex(const CCompileContext& context) const;
   virtual CCompileError* GenerateCode(CCodeSequence& code,
 		   CCompileContext& context, unsigned short& stackUsed,
 	           bool isStatement = false);
@@ -396,6 +399,42 @@ private:
 };
 
 
+//
+//  Class name : COuterLocalExpression
+//  Description : This type of expression is caused by an inner class making
+//    reference to a local variable in the outer class's block.  This is a
+//    little bit of a mess because this expression is constructed before
+//    we get a chance to find a conflicting field on the inner class, so
+//    this may end up having to revise itself.
+//
+class COuterLocalExpression : public CExpression {
+  DynamicCastDeclarations;
+public:
+  COuterLocalExpression(const CVariableDeclaration& declaration,
+			unsigned short localVariableIndex);
+  COuterLocalExpression(const COuterLocalExpression& source);
+  virtual ~COuterLocalExpression();
+  virtual CExpression* Clone() const
+     { return new COuterLocalExpression(*this); }
+
+  unicode_string GetLabel() const { return fDeclaration.GetName(); }
+  CVariableDeclaration GetLocalDeclaration() const { return fDeclaration; }
+  unsigned short GetLocalVariableIndex() const { return fLocalVariableIndex; }
+  bool IsLocalFinal() const { return fDeclaration.IsFinal(); }
+
+  virtual CCompileError* GenerateCode(CCodeSequence& code,
+		   CCompileContext& context, unsigned short& stackUsed,
+	           bool isStatement = false);
+protected:
+  virtual CCompileError* HandleEvaluateType(CCompileContext& context,
+					    CJavaTypeSignature& setType);
+private:
+  CVariableDeclaration fDeclaration;
+  unsigned short fLocalVariableIndex;
+  CExpression* fSurrogate;
+};
+
+
 //
 //  Class name : CArrayInitializer
 //  Description : This type of expression is only used in initialization of
